#!/usr/bin/env python3

# Copyright 2014 Brett Slatkin, Pearson Education Inc.
#
# Udostępniono na licencji Apache w wersji 2.0 ("Licencja").
# Tego pliku można używać jedynie zgodnie z warunkami Licencji.
# Treść Licencji znajdziesz na stronie:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# O ile obowiązujące prawo nie stanowi inaczej lub czegoś innego nie
# uzgodniono w formie pisemnej, oprogramowanie objęte Licencją jest
# dostarczane w stanie, w jakim jest (wersja "AS IS"), BEZ JAKIEJKOLWIEK
# GWARANCJI, ani wyrażonej otwarcie, ani domyślnej. Dokładne zasady
# i warunki Licencji znajdziesz w jej treści.

# Przygotowania mające na celu odtworzenie środowiska użytego w książce.
import logging
from pprint import pprint
from sys import stdout as STDOUT


# Przykład 1.
def sort_priority(values, group):
    def helper(x):
        if x in group:
            return (0, x)
        return (1, x)
    values.sort(key=helper)


# Przykład 2.
numbers = [8, 3, 1, 2, 5, 4, 7, 6]
group = {2, 3, 5, 7}
sort_priority(numbers, group)
print(numbers)


# Przykład 3.
def sort_priority2(numbers, group):
    found = False
    def helper(x):
        if x in group:
            found = True  # To wydaje się być proste.
            return (0, x)
        return (1, x)
    numbers.sort(key=helper)
    return found


# Przykład 4.
found = sort_priority2(numbers, group)
print('Znaleziono:', found)
print(numbers)


# Przykład 5.
def sort_priority2(numbers, group):
    found = False         # Zakres: funkcja sort_priority2().
    def helper(x):
        if x in group:
            found = True  # Zakres: funkcja domknięcia helper() -- nie dobrze!
            return (0, x)
        return (1, x)
    numbers.sort(key=helper)
    return found


# Przykład 6.
def sort_priority3(numbers, group):
    found = False
    def helper(x):
        nonlocal found
        if x in group:
            found = True
            return (0, x)
        return (1, x)
    numbers.sort(key=helper)
    return found


# Przykład 7.
found = sort_priority3(numbers, group)
print('Znaleziono:', found)
print(numbers)


# Przykład 8.
class Sorter(object):
    def __init__(self, group):
        self.group = group
        self.found = False

    def __call__(self, x):
        if x in self.group:
            self.found = True
            return (0, x)
        return (1, x)

sorter = Sorter(group)
numbers.sort(key=sorter)
assert sorter.found is True
print('Znaleziono:', found)
print(numbers)
